<?php

namespace Drupal\burndown\Controller;

use Drupal\Component\Utility\Xss;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Url;
use Drupal\burndown\Entity\TaskInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\burndown\Entity\Task;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Drupal\user\Entity\User;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\JsonResponse;
use Drupal\burndown\Event\TaskCommentEvent;

/**
 * Class TaskController.
 *
 *  Returns responses for Task routes.
 */
class TaskController extends ControllerBase implements ContainerInjectionInterface {

  /**
   * The date formatter.
   *
   * @var \Drupal\Core\Datetime\DateFormatter
   */
  protected $dateFormatter;

  /**
   * The renderer.
   *
   * @var \Drupal\Core\Render\Renderer
   */
  protected $renderer;

  /**
   * Event dispatcher.
   *
   * @var Symfony\Component\EventDispatcher\EventDispatcher
   */
  protected $event_dispatcher;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->dateFormatter = $container->get('date.formatter');
    $instance->renderer = $container->get('renderer');
    $instance->event_dispatcher = $container->get('event_dispatcher');
    return $instance;
  }

  /**
   * Get the log for a task (optionally filtered by type).
   *
   * @param int $ticket_id
   *   The Task ID.
   *
   * @param string $type
   *   The type of log to return.
   *
   * @return array
   *   An array suitable for drupal_render().
   */
  public function get_task_log($ticket_id, $type = 'all') {
    $data = [];

    $task = Task::loadFromTicketID($ticket_id);
    if ($task === FALSE) {
      // Task doesn't exist; throw 404.
      throw new NotFoundHttpException();
    }

    $log = $task->getLog();
    if (!empty($log)) {
      foreach ($log as $log_item) {
        if ($type !== 'all') {
          if ($log_item['type'] !== $type) {
            continue;
          }
        }
        
        // Get user name.
        $user = User::load($log_item['uid']);
        $log_item['user'] = $user->getDisplayName();

        // Date format.
        $created_date = date('r', intval($log_item['created']));
        $log_item['created'] = $created_date;

        $data[] = $log_item;
      }
    }

    $build = [
      '#theme' => 'burndown_log_items',
      '#data' => $data,
    ];

    return new Response(render($build));
  }

  /**
   * Add a comment to a Task.
   *
   * @param int $ticket_id
   *   The Task ID.
   *
   * @param string $comment
   *   The comment (rich HTML).
   */
  public function add_comment( Request $request ) {
    // Get data from request (validated below).
    $ticket_id = $request->request->get('ticket_id');
    $comment = $request->request->get('comment');

    // Load the task.
    $task = Task::loadFromTicketID($ticket_id);
    if ($task === FALSE) {
      // Task doesn't exist; throw 404.
      throw new NotFoundHttpException();
    }

    // Add comment to the log.
    $type = 'comment';
    $filtered_comment = Xss::filter($comment);
    $work_done = '';
    $created = time();
    $uid = \Drupal::currentUser()->id();
    $description = '';

    $task
      ->addLog($type, $filtered_comment, $work_done, $created, $uid, $description)
      ->save();

    // Instantiate our event.
    $event = new TaskCommentEvent($task, $filtered_comment);

    // Get the event_dispatcher service and dispatch the event.
    $this->event_dispatcher->dispatch(TaskCommentEvent::COMMENTED, $event);

    // Return JSON response.
    return new JsonResponse([
      'success' => 1,
      'method' => 'POST',
    ]);
  }

  /**
   * Add a work log to the Task.
   *
   * @param int $ticket_id
   *   The Task ID.
   *
   * @param string $comment
   *   A comment (rich HTML).
   *
   * @param numeric $work
   *   The quantity of work done.
   *
   * @param char $work_increment
   *   A single char describing the period (i.e. 'm' == minutes).
   */
  public static function add_work( Request $request ) {
    // Get data from request (validated below).
    $ticket_id = $request->request->get('ticket_id');
    $comment = $request->request->get('comment');
    $work = $request->request->get('work');
    $work_increment = $request->request->get('work_increment');

    // Load the task.
    $task = Task::loadFromTicketID($ticket_id);
    if ($task === FALSE) {
      // Task doesn't exist; throw 404.
      throw new NotFoundHttpException();
    }

    // Validate the $work is numeric.
    if (!is_numeric($work)) {
      throw new NotFoundHttpException();
    }

    // Validate that $work_increment is one of allowed list.
    $allowed = ['m', 'h', 'd', 'w', 'M', 'Y'];
    if (!in_array($work_increment, $allowed)) {
      throw new NotFoundHttpException();
    }

    // Add work to the log.
    $type = 'work';
    $filtered_comment = Xss::filter($comment);
    $work_done = $work . $work_increment;
    $created = time();
    $uid = \Drupal::currentUser()->id();
    $description = '';

    $task
      ->addLog($type, $filtered_comment, $work_done, $created, $uid, $description)
      ->save();

    // Return JSON response.
    return new JsonResponse([
      'success' => 1,
      'method' => 'POST',
    ]);
  }

  /**
   * Add a user to the task watchlist.
   *
   * @param int $ticket_id
   *   The Task ID.
   *
   * @param int $user_id
   *   The User ID.
   */
  public static function add_to_watchlist($ticket_id, $user_id) {
    // Load the task.
    $task = Task::loadFromTicketID($ticket_id);
    if ($task === FALSE) {
      // Task doesn't exist; throw 404.
      throw new NotFoundHttpException();
    }

    // Load the user.
    $user = User::load($user_id);
    if ($user === FALSE) {
      throw new NotFoundHttpException();
    }

    // Check if this is the current user.
    $current_user = \Drupal::currentUser();
    if ($current_user->id() !== $user_id) {
      // Only allow admins to subscribe somebody else.
      $current_user_roles = $current_user->getRoles();
      if (!in_array('administrator', $current_user_roles)) {
        throw new NotFoundHttpException();
      }
    }

    // Add to watchlist.
    $task->addToWatchlist($user)->save();

    // Return JSON response.
    return new JsonResponse([
      'ticket_id' => $ticket_id,
      'user_id' => $user_id,
      'success' => 1,
      'method' => 'POST',
    ]);
  }

  /**
   * Remove a user from the task watchlist.
   *
   * @param int $ticket_id
   *   The Task ID.
   *
   * @param int $user_id
   *   The User ID.
   */
  public static function remove_from_watchlist($ticket_id, $user_id) {
    // Load the task.
    $task = Task::loadFromTicketID($ticket_id);
    if ($task === FALSE) {
      // Task doesn't exist; throw 404.
      throw new NotFoundHttpException();
    }

    // Load the user.
    $user = User::load($user_id);
    if ($user === FALSE) {
      throw new NotFoundHttpException();
    }

    // Check if this is the current user.
    $current_user = \Drupal::currentUser();
    if ($current_user->id() !== $user_id) {
      // Only allow admins to unsubscribe somebody else.
      $current_user_roles = $current_user->getRoles();
      if (!in_array('administrator', $current_user_roles)) {
        throw new NotFoundHttpException();
      }
    }

    // Remove from watchlist.
    $task->removeFromWatchlist($user);

    // Return JSON response.
    return new JsonResponse([
      'ticket_id' => $ticket_id,
      'user_id' => $user_id,
      'success' => 1,
      'method' => 'POST',
    ]);
  }

  /**
   * Displays a Task revision.
   *
   * @param int $burndown_task_revision
   *   The Task revision ID.
   *
   * @return array
   *   An array suitable for drupal_render().
   */
  public function revisionShow($burndown_task_revision) {
    $burndown_task = $this->entityTypeManager()->getStorage('burndown_task')
      ->loadRevision($burndown_task_revision);
    $view_builder = $this->entityTypeManager()->getViewBuilder('burndown_task');

    return $view_builder->view($burndown_task);
  }

  /**
   * Page title callback for a Task revision.
   *
   * @param int $burndown_task_revision
   *   The Task revision ID.
   *
   * @return string
   *   The page title.
   */
  public function revisionPageTitle($burndown_task_revision) {
    $burndown_task = $this->entityTypeManager()->getStorage('burndown_task')
      ->loadRevision($burndown_task_revision);
    return $this->t('Revision of %title from %date', [
      '%title' => $burndown_task->label(),
      '%date' => $this->dateFormatter->format($burndown_task->getRevisionCreationTime()),
    ]);
  }

  /**
   * Generates an overview table of older revisions of a Task.
   *
   * @param \Drupal\burndown\Entity\TaskInterface $burndown_task
   *   A Task object.
   *
   * @return array
   *   An array as expected by drupal_render().
   */
  public function revisionOverview(TaskInterface $burndown_task) {
    $account = $this->currentUser();
    $burndown_task_storage = $this->entityTypeManager()->getStorage('burndown_task');

    $langcode = $burndown_task->language()->getId();
    $langname = $burndown_task->language()->getName();
    $languages = $burndown_task->getTranslationLanguages();
    $has_translations = (count($languages) > 1);
    $build['#title'] = $has_translations ? $this->t('@langname revisions for %title', ['@langname' => $langname, '%title' => $burndown_task->label()]) : $this->t('Revisions for %title', ['%title' => $burndown_task->label()]);

    $header = [$this->t('Revision'), $this->t('Operations')];
    $revert_permission = (($account->hasPermission("revert all task revisions") || $account->hasPermission('administer task entities')));
    $delete_permission = (($account->hasPermission("delete all task revisions") || $account->hasPermission('administer task entities')));

    $rows = [];

    $vids = $burndown_task_storage->revisionIds($burndown_task);

    $latest_revision = TRUE;

    foreach (array_reverse($vids) as $vid) {
      /** @var \Drupal\burndown\TaskInterface $revision */
      $revision = $burndown_task_storage->loadRevision($vid);
      // Only show revisions that are affected by the language that is being
      // displayed.
      if ($revision->hasTranslation($langcode) && $revision->getTranslation($langcode)->isRevisionTranslationAffected()) {
        $username = [
          '#theme' => 'username',
          '#account' => $revision->getRevisionUser(),
        ];

        // Use revision link to link to revisions that are not active.
        $date = $this->dateFormatter->format($revision->getRevisionCreationTime(), 'short');
        if ($vid != $burndown_task->getRevisionId()) {
          $link = $this->l($date, new Url('entity.burndown_task.revision', [
            'burndown_task' => $burndown_task->id(),
            'burndown_task_revision' => $vid,
          ]));
        }
        else {
          $link = $burndown_task->link($date);
        }

        $row = [];
        $column = [
          'data' => [
            '#type' => 'inline_template',
            '#template' => '{% trans %}{{ date }} by {{ username }}{% endtrans %}{% if message %}<p class="revision-log">{{ message }}</p>{% endif %}',
            '#context' => [
              'date' => $link,
              'username' => $this->renderer->renderPlain($username),
              'message' => [
                '#markup' => $revision->getRevisionLogMessage(),
                '#allowed_tags' => Xss::getHtmlTagList(),
              ],
            ],
          ],
        ];
        $row[] = $column;

        if ($latest_revision) {
          $row[] = [
            'data' => [
              '#prefix' => '<em>',
              '#markup' => $this->t('Current revision'),
              '#suffix' => '</em>',
            ],
          ];
          foreach ($row as &$current) {
            $current['class'] = ['revision-current'];
          }
          $latest_revision = FALSE;
        }
        else {
          $links = [];
          if ($revert_permission) {
            $links['revert'] = [
              'title' => $this->t('Revert'),
              'url' => $has_translations ?
              Url::fromRoute('entity.burndown_task.translation_revert', [
                'burndown_task' => $burndown_task->id(),
                'burndown_task_revision' => $vid,
                'langcode' => $langcode,
              ]) :
              Url::fromRoute('entity.burndown_task.revision_revert', [
                'burndown_task' => $burndown_task->id(),
                'burndown_task_revision' => $vid,
              ]),
            ];
          }

          if ($delete_permission) {
            $links['delete'] = [
              'title' => $this->t('Delete'),
              'url' => Url::fromRoute('entity.burndown_task.revision_delete', [
                'burndown_task' => $burndown_task->id(),
                'burndown_task_revision' => $vid,
              ]),
            ];
          }

          $row[] = [
            'data' => [
              '#type' => 'operations',
              '#links' => $links,
            ],
          ];
        }

        $rows[] = $row;
      }
    }

    $build['burndown_task_revisions_table'] = [
      '#theme' => 'table',
      '#rows' => $rows,
      '#header' => $header,
    ];

    return $build;
  }

}
